
(** This is the main prover engine. It implements the inversion proof 
 * procedure to destruct predicates into atoms and then drives the 
 * satisfiability procedures. Finally it constructs the proof *)


val debug : bool ref

val continue_on_error: bool ref
val continue_without_asking: bool ref

val timingFlag: bool ref

type state                              (** The state of the prover *)

type proofgen =                         (** Something to process the proofs *)
    { start: unit -> unit;
      startVC: string -> unit;
      assume: Util.symbol -> Logic.pred -> unit;
      unassume: unit -> unit;
      pick: string -> unit;
      unpick: unit -> unit;
      conjunct: Logic.pred -> Proof.proof -> unit;
      choose: thechoice:int -> nrchoices:int -> unit;  
      finish: unit -> unit;
      finishVC: unit -> unit
    }

                                        (** Start a set of proofs *)
val start: Satproc.entries list -> proofgen -> state

(** Prove a goal, given a name, a state, a function for reading the elements 
 * of the goal *)
val proveGoal: string -> state -> (unit -> Input.input) -> unit

                                        (** Finish a set of proofs *)
val finish: state -> unit
                                        (** Raised when proving fails *)
exception Failure


class type uiType = object
  method name: string
  method options: Options.optionDescr list
  method enterBlock: (Util.symbol * Logic.pred) list -> 
                     (string * string) list -> unit
  method exitBlock: unit

  (** Called when a goal is initiated *)
  method start_goal: Logic.pred -> unit
      

  (** Called when a goal is successfully proved *)
  method goal_success: Logic.pred -> Proof.proof -> unit

  (** Called when a goal fails *)
  method goal_failure: Logic.pred -> unit

  (** Called when an assumption is encountered *)
  method assume: Util.symbol -> Logic.pred -> unit

  (** Called when an assumption is retracted *)
  method unassume: unit

  (** Called to start the UI *)  
  method run: unit
end

val theUI: uiType ref


(** Define new printers that print either infix or prefix *)
val printInfix: bool ref
val d_exp: unit -> Logic.exp -> Pretty.doc
val d_pred: unit -> Logic.pred -> Pretty.doc
