open Pretty
module E = Errormsg
open Logic


type axiom = string * var list * pred list * pred
(** The input to the prover is presented as a sequence of "inputs".*)
type input =
    (** The next few kinds of inputs can appear only at top level *)
    StartGoal of string                 (* The start of a goal. Followed by a 
                                         * number of elements ended by 
                                         * EndGoal *)
  | TopChoose of int * int              (* A choose at the top level *)
  | Axiom  of axiom
  | Import of string
  | Decl   of string
  | Eof

  (** The next few kinds of inputs can appear only after a StartGoal **)

  | Prove of Logic.pred (* Prove something. Not a conjunction, nor 
                         * implication, nor universal quantification. *)

  | Assume of Logic.pred (* Assume something. Left-hand of an implication *)
  | UnAssume       (* retract the last assumption *)

  | Pick of string    (* Pick a variable. Universal quantification *)
  | UnPick         (* Retract the last variable picked *)

  | Choose of int * int
  | Block of (string * string) list
  | UnBlock
  | Comment of string
  | EndGoal



(********* Pretty printing ***********)
let printLFSyntax = ref false

class lfLogicPrinterClass: logicPrinter = object (self)
    inherit defaultLogicPrinterClass

end

let lfLogicPrinter = new lfLogicPrinterClass

let di () = function
    StartGoal name -> dprintf "StartGoal(%s)" name
  | EndGoal -> dprintf "EndGoal"
  | Prove p -> dprintf "Prove(@[%a@])" d_pred p
  | Choose(i,nr) -> dprintf "Choose(%d, %d)" i nr
  | Comment s -> dprintf "%% %s" s
  | Block sl -> dprintf "Block [%a]" 
                    (docList
                        (fun (s, s') -> dprintf "%s=\"%s\"" s s'))
                    sl
  | UnBlock -> text "UnBlock"
  | Assume p -> dprintf "Assume(@[%a@])" d_pred p
  | UnAssume -> text "UnAssume"
  | Pick v -> dprintf "Pick(%s)" v
  | UnPick -> text "UnPick"
  | Decl s -> dprintf "Decl(%s)" s
  | Import s ->  dprintf "Import(%s)" s
  | Eof -> text "Eof"
  | Axiom (n, _, _, _) -> dprintf "Axiom(%s)" n
  | TopChoose (n, m) -> dprintf "TopChoose(%d, %d)" n m



