
(* This defines data types for the output of the parser. *)

type axiom = string * string list * Logic.pred list * Logic.pred

(** The input to the prover is presented as a sequence of "inputs".*)
type input =
    (** The next few kinds of inputs can appear only at top level *)
    StartGoal of string                 (* The start of a goal. Followed by a 
                                         * number of elements ended by 
                                         * some top-level element *)
  | TopChoose of int * int              (* A choose at the top level *)
  | Axiom  of axiom
  | Import of string
  | Decl   of string
  | Eof

  (** The next few kinds of inputs can appear only after a StartGoal **)

  | Prove of Logic.pred (* Prove something. Not a conjunction, nor 
                         * implication, nor universal quantification. *)

  | Assume of Logic.pred (* Assume something. Left-hand of an implication *)
  | UnAssume       (* retract the last assumption *)

  | Pick of string    (* Pick a variable. Universal quantification *)
  | UnPick         (* Retract the last variable picked *)

  | Choose of int * int
  | Block of (string * string) list
  | UnBlock
  | Comment of string
  | EndGoal


val di  : unit -> input -> Pretty.doc

val printLFSyntax : bool ref            (* If true then things are printed in 
                                         * the prefix notation of LF *)


