open Pretty

(* This file generates a set of (n+1) linear equations in n variables *)



(* Tunable parameters *)

let nrEq = ref (20) (* number of equations *)
(* nrEq should be less than 26, since the size of the set of English alphabets is 26 *)
let randBound = ref (10) (* limit on the size of random numbers *)
let eqFileName = ref ("eq.p") (* name of the output file *)



let randNum () = Random.int !randBound
let charI i = Char.chr ((Char.code 'a') + i)


let main = 
  let usageMsg = "Usage: eqgen [options]" in
  let argDescr = [
    "-n", Arg.Int (fun i -> if (i > 26) then nrEq := 26 else nrEq := i), "the number of equations";
    "-r", Arg.Int (fun i -> randBound := i), "the range of random numbers";
    "-o", Arg.String (fun s -> eqFileName := s), "the name of the output file";
  ] in
  Arg.parse argDescr (fun s -> ()) usageMsg;
  Random.self_init ();
  let nrVar = !nrEq in
  let values = Array.init nrVar (fun i -> randNum ()) in
  let equations = Array.init (nrVar + 1) (fun i -> Array.init (nrVar + 1) (fun j -> randNum ())) in
  for i = 0 to nrVar do
    let result = ref (0) in
    for j = 0 to nrVar - 1 do
      let term = values.(j) * equations.(i).(j) in
      result := !result + term;
    done;
    equations.(i).(nrVar) <- !result;
  done;

  let rec docEquations () i = 
    let rec docLHS () j =
      if j = (nrVar-1) then dprintf "* %d %c" equations.(i).(j) (charI j)
      else dprintf "+ (* %d %c) (%a)" equations.(i).(j) (charI j) docLHS (j+1)
    in
    if (i = nrVar) then dprintf "(= %a %d)" docLHS 0 (equations.(i).(nrVar))
    else dprintf "(=> (= %a %d) @@! %a)" docLHS 0 (equations.(i).(nrVar)) docEquations (i+1)
  in
  let solution = dprintf "%a" (docArray (fun i v -> dprintf "%c = %d " (charI i) v)) values in 
  let content = dprintf "ANN_STARTLF\n%% %a\nP1 :pf %a.\nANN_ENDLF" insert solution docEquations 0 in
  let eqFile = open_out !eqFileName in
  ignore (fprintf eqFile "%a" insert content)
